@@ -1,31 +1,31 @@
 # as defined by the language server protocol
 SymbolKind <- list(
-	 File = 1,
-	 Module = 2,
-	 Namespace = 3,
-	 Package = 4,
-	 Class = 5,
-	 Method = 6,
-	 Property = 7,
-	 Field = 8,
-	 Constructor = 9,
-	 Enum = 10,
-	 Interface = 11,
-	 Function = 12,
-	 Variable = 13,
-	 Constant = 14,
-	 String = 15,
-	 Number = 16,
-	 Boolean = 17,
-	 Array = 18,
-	 Object = 19,
-	 Key = 20,
-	 Null = 21,
-	 EnumMember = 22,
-	 Struct = 23,
-	 Event = 24,
-	 Operator = 25,
-	 TypeParameter = 26
+    File = 1,
+    Module = 2,
+    Namespace = 3,
+    Package = 4,
+    Class = 5,
+    Method = 6,
+    Property = 7,
+    Field = 8,
+    Constructor = 9,
+    Enum = 10,
+    Interface = 11,
+    Function = 12,
+    Variable = 13,
+    Constant = 14,
+    String = 15,
+    Number = 16,
+    Boolean = 17,
+    Array = 18,
+    Object = 19,
+    Key = 20,
+    Null = 21,
+    EnumMember = 22,
+    Struct = 23,
+    Event = 24,
+    Operator = 25,
+    TypeParameter = 26
 )
 
 fuzzy_find <- function(pattern, x) {
@@ -110,32 +110,32 @@ find_definition_in_package <- function(workspace, funct, pkg) {
 #'
 #' @keywords internal
 definition_reply <- function(id, uri, workspace, document, position) {
-    token <- detect_hover(document, position)
 
-    logger$info("definition: ", token$text)
+    token_result <- document$detect_token(position)
 
-    matches <- stringr::str_match(
-        token$text, "(?:([a-zA-Z][a-zA-Z0-9.]+)(:::?))?([a-zA-Z0-9_.]*)$")
-    pkg <- matches[2]
-    funct <- matches[4]
+    pkg <- token_result$package
+    token <- token_result$token
 
-    if (is.na(pkg)) {
-        # look for in file first
-        definition <- workspace$get_definition(funct)
+    if (is.null(pkg)) {
+        # look in file first
+        definition <- workspace$get_definition(token)
         result <- if (is.null(definition)) {
-            find_definition_in_package(workspace, funct, pkg)
+            find_definition_in_package(workspace, token, pkg)
         } else {
             definition
         }
     } else {
-        # look for in package first
-        definition <- find_definition_in_package(workspace, funct, pkg)
+        # then look in package
+        definition <- find_definition_in_package(workspace, token, pkg)
         result <- if (is.null(definition)) {
-            workspace$get_definition(funct)
+            workspace$get_definition(token)
         } else {
             definition
         }
     }
+
+    logger$info("definition", definition)
+
     if (is.null(result)) {
         Response$new(id)
     } else {
