@@ -1,6 +1,6 @@
 #' textDocument/didOpen notification handler
 #'
-#' Handler to the [textDocument/didOpen](https://microsoft.github.io/language-server-protocol/specification#textDocument_didOpen) [Notification]
+#' Handler to the [textDocument/didOpen](https://microsoft.github.io/language-server-protocol/) [Notification]
 #'
 #' @template self
 #' @param params a [did_open_text_document_params]
@@ -9,14 +9,18 @@
 text_document_did_open <- function(self, params) {
     textDocument <- params$textDocument
     uri <- textDocument$uri
-    doc <- readLines(path_from_uri(uri), warn = FALSE)
-    self$documents[[uri]] <- doc
+    content <- readLines(path_from_uri(uri), warn = FALSE)
+    if (is.null(self$documents[[uri]])) {
+        self$documents[[uri]] <- Document$new(uri, content)
+    } else {
+        self$documents[[uri]]$set(content)
+    }
     self$text_sync(uri, document = NULL, run_lintr = TRUE, parse = TRUE)
 }
 
 #' textDocument/didChange notification handler
 #'
-#' Handler to the [textDocument/didChange](https://microsoft.github.io/language-server-protocol/specification#textDocument_didChange) [Notification]
+#' Handler to the [textDocument/didChange](https://microsoft.github.io/language-server-protocol/) [Notification]
 #'
 #' @template self
 #' @param params a [did_change_text_document_params]
@@ -28,16 +32,20 @@ text_document_did_change <- function(self, params) {
     text <- contentChanges[[1]]$text
     uri <- textDocument$uri
     logger$info("did change: ", uri)
-    doc <- stringr::str_split(text, "\r\n|\n")[[1]]
-    # remove last empty line
-    if (nchar(doc[[length(doc)]]) == 0) doc <- doc[-length(doc)]
-    self$documents[[uri]] <- doc
+    content <- stringr::str_split(text, "\r\n|\n")[[1]]
+    if (is.null(self$documents[[uri]])) {
+        doc <- Document$new(uri, content)
+        self$documents[[uri]] <- doc
+    } else {
+        self$documents[[uri]]$set(content)
+        doc <- self$documents[[uri]]
+    }
     self$text_sync(uri, document = doc, run_lintr = TRUE, parse = FALSE)
 }
 
 #' textDocument/willSave notification handler
 #'
-#' Handler to the [textDocument/willSave](https://microsoft.github.io/language-server-protocol/specification#textDocument_willSave) [Notification]
+#' Handler to the [textDocument/willSave](https://microsoft.github.io/language-server-protocol/) [Notification]
 #'
 #' @template self
 #' @param params a [will_save_text_document_params]
@@ -49,7 +57,7 @@ text_document_will_save <- function(self, params) {
 
 #' textDocument/didSave notification handler
 #'
-#' Handler to the [textDocument/didSave](https://microsoft.github.io/language-server-protocol/specification#textDocument_didSave) [Notification]
+#' Handler to the [textDocument/didSave](https://microsoft.github.io/language-server-protocol/) [Notification]
 #'
 #' @template self
 #' @param params a [did_save_text_document_params]
@@ -59,13 +67,14 @@ text_document_did_save <- function(self, params) {
     textDocument <- params$textDocument
     uri <- textDocument$uri
     logger$info("did save:", uri)
-    self$documents[[uri]] <- readLines(path_from_uri(uri), warn = FALSE)
+    content <- readLines(path_from_uri(uri), warn = FALSE)
+    self$documents[[uri]] <- Document$new(uri, content)
     self$text_sync(uri, document = NULL, run_lintr = TRUE, parse = TRUE)
 }
 
 #' textDocument/didClose notification handler
 #'
-#' Handler to the [textDocument/didClose](https://microsoft.github.io/language-server-protocol/specification#textDocument_didClose) [Notification]
+#' Handler to the [textDocument/didClose](https://microsoft.github.io/language-server-protocol/) [Notification]
 #'
 #' @template self
 #' @param params a [did_close_text_document_params]
@@ -79,7 +88,7 @@ text_document_did_close <- function(self, params) {
 
 #' textDocument/willSaveWaitUntil notification handler
 #'
-#' Handler to the [textDocument/willSaveWaitUntil](https://microsoft.github.io/language-server-protocol/specification#textDocument_willSaveWaitUntil) [Request]
+#' Handler to the [textDocument/willSaveWaitUntil](https://microsoft.github.io/language-server-protocol/) [Request]
 #'
 #' @template self
 #' @template id
